/****************************************************************************
** $Id: globjwin.cpp,v 1.3 2002/02/08 14:53:35 tim Exp $
**
** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
**
** This file is part of an example program for Qt.  This example
** program may be used, distributed and modified without limitation.
**
*****************************************************************************/

#include <qpushbutton.h>
#include <qslider.h>
#include <qlayout.h>
#include <qframe.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qapplication.h>
#include <qkeycode.h>
#include "globjwin.h"
#include "gltexobj.h"


GLObjectWindow::GLObjectWindow( QWidget* parent, const char* name )
    : QWidget( parent, name )
{
	setCaption("Tree - an OpenGL tree");

	// Create nice frames to put around the OpenGL widgets
	QFrame* f1 = new QFrame( this, "frame1" );
	f1->setFrameStyle( QFrame::Sunken | QFrame::Panel );
	f1->setLineWidth( 2 );

	// Create an OpenGL widget
	GLTexobj* c = new GLTexobj( f1, "glbox1");

	// Create a menu
	QPopupMenu *file = new QPopupMenu( this );
	file->insertItem( "E&xit",  qApp, SLOT(quit()), CTRL+Key_Q );

	 // Create a menu
	QPopupMenu *tree = new QPopupMenu( this );
	tree->setCheckable(true);
	tree->insertItem( "Toggle &Rotating", c, SLOT(toggleRotating()), CTRL+Key_R );
	tree->insertItem( "Toggle &Zooming", c, SLOT(toggleZooming()), CTRL+Key_Z );
	tree->insertItem( "Toggle &Growing", c, SLOT(toggleGrowing()), CTRL+Key_G );
	tree->insertItem( "Toggle &Leaves", c, SLOT(toggleLeaves()), CTRL+Key_L );
	tree->insertItem( "Toggle &Autumn", c, SLOT(toggleAutumn()), CTRL+Key_A );
	tree->insertItem( "Toggle &Wind", c, SLOT(toggleWind()), CTRL+Key_W );
	tree->insertItem( "Toggle &Fog", c, SLOT(toggleFog()), CTRL+Key_F );
	tree->insertSeparator();
	tree->insertItem( "&New tree", c, SLOT(newTree()), CTRL+Key_N );

	// Create a menu bar
	QMenuBar *m = new QMenuBar( this );
	m->setSeparator( QMenuBar::InWindowsStyle );
	m->insertItem("&File", file );
	m->insertItem("&Tree", tree );

	// Create the three sliders; one for each rotation axis
	QSlider* x = new QSlider ( 0, 360, 60, 0, QSlider::Vertical, this, "xsl" );
	x->setTickmarks( QSlider::Left );
	connect( x, SIGNAL(valueChanged(int)), c, SLOT(setXRotation(int)) );

	QSlider* y = new QSlider ( 0, 360, 60, 0, QSlider::Vertical, this, "ysl" );
	y->setTickmarks( QSlider::Left );
	connect( y, SIGNAL(valueChanged(int)), c, SLOT(setYRotation(int)) );

	QSlider* z = new QSlider ( 0, 360, 60, 0, QSlider::Vertical, this, "zsl" );
	z->setTickmarks( QSlider::Left );
	connect( z, SIGNAL(valueChanged(int)), c, SLOT(setZRotation(int)) );


	// Now that we have all the widgets, put them into a nice layout

	// Put the sliders on top of each other
	QVBoxLayout* vlayout = new QVBoxLayout( 20, "vlayout");
	vlayout->addWidget( x );
	vlayout->addWidget( y );
	vlayout->addWidget( z );

	// Put the GL widget inside the frame
	QHBoxLayout* flayout1 = new QHBoxLayout( f1, 2, 2, "flayout1");
	flayout1->addWidget( c, 1 );

	// Top level layout, puts the sliders to the left of the frame/GL widget
	QHBoxLayout* hlayout = new QHBoxLayout( this, 20, 20, "hlayout");
	hlayout->setMenuBar( m );
	hlayout->addLayout( vlayout );
	hlayout->addWidget( f1, 1 );

}
