/****************************************************************************
** $Id: gltexobj.h,v 1.5 2002/02/08 14:53:35 tim Exp $
**
** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
**
** This file is part of an example program for Qt.  This example
** program may be used, distributed and modified without limitation.
**
*****************************************************************************/

/****************************************************************************
**
** This is a simple QGLWidget displaying an openGL wireframe box
**
****************************************************************************/

#ifndef GLTEXOBJ_H
#define GLTEXOBJ_H

#include <stdlib.h>

#include <qgl.h>

class Tree;


class GLTexobj : public QGLWidget
{
	Q_OBJECT

public:

	GLTexobj( QWidget* parent, const char* name );
	~GLTexobj();

public slots:

	void	setXRotation( int degrees );
	void	setYRotation( int degrees );
	void	setZRotation( int degrees );
	void	toggleRotating();
	void	toggleZooming();
	void	toggleGrowing();
	void	toggleLeaves();
	void	toggleAutumn();
	void	toggleWind();
	void	toggleFog();
	void	newTree();

protected:

	void		initializeGL();
	void		paintGL();
	void		resizeGL( int w, int h );

private:
	// menu slot vars for animations
	bool rotating;
	GLfloat xRot, yRot, zRot;
	float rotoff;
	bool xrotdir;
	bool zooming;
	bool zoom;	// zoom in (i.e. enlarge) if true, else shrink
	GLfloat scale;
	int max_age;	// age of tree when "full-grown"
	float age;		// current age of tree
	bool growing;	// or shrinking, i.e. true if size is changing
	bool grow;	// else shrink
	bool show_leaves;
	bool autumn;	// leaves going/are brown
	int season;	// when reaches 0, season reached
	bool wind;
	bool fog;

	GLuint object;
	QTimer* timer;

	GLuint texnum_sky;
	GLuint texnum_grass;

	Tree* tree;

	bool animating() { return rotating || zooming || growing || season != 0 || wind; }

	void draw_sky();
	void draw_grass();

	int rndr(int range) { return 1 + (int)((float(range)*rand()/(RAND_MAX+1.0))); }	// random number in 1..range
	int rndb(int range) { return (int)((float(range*2+1)*rand()/(RAND_MAX+1.0))) - range; }	// random num in -range..range

};


#endif // GLTEXOBJ_H
