/***************************************************************************
                          tree.h

Abstract base class for a tree. The interface that needs to be implemented in
the derived class is pretty narrow, basically all that is required is to
calculate all the fixed values in the constructor, and then over-ride the
draw_tree() method so actually draw the tree. This method should check the
protected variables: wind, show_leaves, colour, current_age etc. to adjust its
drawing - see tree1.cpp for an example of this.

                    -------------------
    begin                : Wed Oct 31 2001
    copyright            : (C) 2001 by tim
    email                : tim@simply.tim
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TREE_H
#define TREE_H

#include <stdlib.h>		// rand()
#include <math.h>

/**represents a tree (i.e. branches and twigs)
  *@author tim
  * this is the abstract base class; actual trees are sub-classed.
  */

class Tree {
public: 
	Tree(int age_, float len_);
	virtual ~Tree();
	
	virtual void draw_tree(float yrs=-1)=0;		// if (yrs < age) draw tree as it was when it was younger
	void setLeaves(bool on) { show_leaves = on; }
	// change this to setSeason: 0=start of Spring .. 1 = end of Winter
	void setColour(float amount) { colour = amount; }	// 0 = brown, 1 = green, 0.5 = half-way between
	void setWind(bool on) { wind = on; }

protected:
	// these variables are fixed for the lifetime of the tree
	const int age;	// age of tree in years, controls depth of branching
	const float len;	// length of first year's trunk. age*len gives indication of overall size of tree

	float current_age;	// used for drawing routines - can draw tree as it was when it was younger
	float year_frac;	// fraction of the current year's growth to display
	bool show_leaves;
	float colour;
	bool wind;
	
	// general utility functions
	float age_now(int a) { return a - (age - current_age); }
	int age_nowi(int a) { return a - (age - (int)current_age); }
	int rndr(int range) { return 1 + (int)((float(range)*rand()/(RAND_MAX+1.0))); }	// random number in 1..range
	int rndb(int range) { return (int)((float(range*2+1)*rand()/(RAND_MAX+1.0))) - range; }	// random num in -range..range
	float torad(int deg) { return deg * 3.14159 / 180.0; }
};


extern const int pregen_random[256];

// some convenient maths functions. May not be 100% exact, but hopefully much quicker.
class QMaths {
public:
	static void srand(int start) { index = start & 255; }		// optional - start random sequence from known position
	static int rndr(int range) { index = index+1 & 255; return 1 + pregen_random[index] % range;  }	// random number in 1..range
	static int rndb(int range) { index = index+1 & 255; return pregen_random[index] % (range*2+1) - range; }	// random num in -range..range

	static void init_sines() {	// a bit crap
		if (init) return;
		for (int i=0; i<360; i++) sines[i] = ::sin(i * 3.14159265/180.0);
		init = true;
	}
	static float sin(int angle) { angle %= 360; if (angle < 0) angle += 360; return sines[angle]; }
	static float cos(int angle) { return sin(angle + 90); }

private:
	static bool init;
	static int index;
	static float sines[360];
};

#endif
