/***************************************************************************
                          tree1.cpp  -  description
                             -------------------
    begin                : Wed Oct 31 2001
    copyright            : (C) 2001 by tim
    email                : tim@mettalogic.uklinux.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <iostream>
#include <stdio.h>	// diagnostics only

#include <qgl.h>
#include <GL/glu.h>

#include "tree1.h"

Tree1::Tree1(int age_, float len_): Tree(age_, len_)
{
	// Set up the bark texture
	QImage buf;

	if ( !buf.load( "bark2.bmp" ) ) {	// Load first image from file
 		cerr << "Could not read image file, using singe-color instead.";
		exit(1);
	}
	tex1 = QGLWidget::convertToGLFormat( buf );  // flipped 32bit RGBA

	glGenTextures(1, &texnum);
	glBindTexture(GL_TEXTURE_2D, texnum);
	glTexImage2D( GL_TEXTURE_2D, 0, 3, tex1.width(), tex1.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, tex1.bits() );

	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

	branch = new_trunk(age, len);
}

Tree1::~Tree1()
{
}


Branch* Tree1::new_branch(int age, float len, int angle, int depth)
{
	if (age <= 0) return NULL;
	if (len <= 0.01) return NULL;	// don't waste time drawing very small twigs

	Branch* b = new Branch;
	b->len = len;
//	b->thick = depth > 1 ? 0.02 : 0.02 + age*0.006;
	b->trunk = false;
	b->depth = depth;
	b->age = age;
	b->random = rndr(256)-1;
	// the final angle is worked out when the branch is drawn so angle can change depending on age of branch
	// just store the random adjustment here
	b->angle_vert = rndb(25);
	b->angle_horiz = angle;

	if (depth > 1) return b;	// don't grow branches off branches off branches

	b->add_twig(new_branch(age-1, len*0.98, angle + rndb(5), depth));	// angle is near 0 for second gen branches - gives near straight branches

	// and two short branches at 90 degrees
	float blen = 0.3 + rndr(50)/100.0;
	b->add_twig(new_branch(age-1, blen, angle - 90, depth+1));
	b->add_twig(new_branch(age-1, blen, angle + 90, depth+1));

	return b;
}

Branch* Tree1::new_trunk(int age, float len)
{
	if (age <= 0) return NULL;
	if (len <= 0.01) return NULL;	// don't waste time drawing very small twigs
	
	Branch* b = new Branch;
	b->len = age == this->age ? len * 1.7 : len*0.98;	// make first segment of trunk longer (not strictly necessary but looks better)
//	b->thick = 0.025 + age*0.009;
	b->trunk = true;
	b->depth = 0;
	b->age = age;
	b->angle_vert = 90;
	b->angle_horiz = 0;
	b->random = rndr(256)-1;

	b->add_twig(new_trunk(age-1, len));
	
	int num = 3 + rndr(3);
//	num=1;
	float start_angle = rndr(360);
	for (int i = 0; i < num; i++)
		b->add_twig(new_branch(age-1, len/2, start_angle + 360/num*i + rndb(25), 1));

	return b;
}

// draw some pine needles
void Tree1::draw_leaves(Branch* b)
{
	const float greens[][3] = {
		{ 0.2, 0.8, 0.3 },
		{ 0.1, 0.4, 0.1 },
		{ 0.1, 0.6, 0.2 },
		{ 0.2, 0.7, 0.1 },
		{ 0.1, 0.3, 0.0 },
		{ 0.0, 0.5, 0.1 }
	};
	
	// must be same number of browns as greens above
	const float browns[][3] = {
		{ 0.6, 0.4, 0.3 },
		{ 0.7, 0.5, 0.3 },
		{ 0.5, 0.3, 0.1 },
		{ 0.5, 0.3, 0.1 },
		{ 0.3, 0.2, 0.1 },
		{ 0.6, 0.4, 0.2 }
	};
	
	const int num_greens = sizeof(greens) / sizeof(greens[0]);
	int g=0;
	float llen = len * 0.4;		// leaf length proportional to first segment of trunk
	if (b->depth > 1 && age_nowi(b->age) == 0) llen *= year_frac;
	
// 	if (b->angle_horiz%360 > 90) return;
	glDisable( GL_TEXTURE_2D );
	glDisable(GL_LIGHTING);
	glLineWidth( 1.0f );
	glBegin(GL_LINES);
	int i2, j2;
	int xspread = b->trunk ? 180 : 54;
// 	xspread=0;
	for (int i= b->angle_horiz - xspread; i<= b->angle_horiz + xspread; i+=18) { 	// spread of angles horizontally
		for (int j=0; j <= 6; j++) {	// spread of angles vertically
			
			g = QMaths::rndr(num_greens) - 1;
			glColor3f(browns[g][0] + (greens[g][0] - browns[g][0]) * colour,
				browns[g][1] + (greens[g][1] - browns[g][1]) * colour,
				browns[g][2] + (greens[g][2] - browns[g][2]) * colour);

			glVertex3f(0.0, 0.0, 0.0);
			i2 = i + QMaths::rndb(15);
			if (wind)
				j2 = j*10 + QMaths::rndb(15);		// needles more horizontal when windy
			else
				j2 = 10 + j*10 + QMaths::rndr(15);
// 		i2 = i; j2 = j*15;
			glVertex3f(llen * QMaths::sin(i2) * QMaths::cos(j2), -llen * QMaths::sin(j2), llen * QMaths::cos(i2) * QMaths::cos(j2));
		}
	}
	glEnd();
	glEnable(GL_LIGHTING);
	glEnable( GL_TEXTURE_2D );
}

//  draw a branch using a texture-mapped cylinder
void Tree1::drawBranchObject( float len, float thick )
{
	float r=thick;
	float l=len;
	int sl=12;	// slices aka smoothness of curves

	glBindTexture( GL_TEXTURE_2D, texnum );

	GLUquadricObj* q = gluNewQuadric();

	gluQuadricTexture( q, GL_TRUE );

	// the default cylinder is drawn with increasing Z, i.e. coming out of the screen
	gluCylinder(q, r, r, l, sl, 1);	// , baseRadius, topRadius, height, slices, stacks

	glTranslatef( 0.0, 0.0, l);	// leave current position at end of cylinder

	gluDeleteQuadric( q );
}

void Tree1::draw_branch(Branch* b)
{
	int hoff=0;	// variation in horizontal angle if windy
	
	if (b == NULL) return;
	if (age_nowi(b->age) < 0) return;
	
	if (wind) hoff = rndb(age - b->age);	// young branches bend more
	
	float angle_vert;
	float thick;
	float len = age_nowi(b->age) == 0 ? b->len * year_frac : b->len;
	// branches grow 2/3 of final length in first year, remaining 1/3 over following years
	len = len*0.67 + len*0.33*age_now(b->age)/b->age;

	if (b->trunk)	{	// draw vertically
		angle_vert =  b->angle_vert;
		thick = 0.025 + age_now(b->age)*0.009;
	}
	else {
		angle_vert =  int(70.0/(age_now(b->age)+.01) * (1 + b->angle_vert/100.0));	// young branches are more vertical, after a few years practically flat
		if (angle_vert > 80) angle_vert = 80;
		thick = b->depth > 1 ? 0.02 : 0.02 + age_now(b->age)*0.006;
	}

	glPushMatrix();

	glRotatef(b->angle_horiz + hoff, 0, 1, 0 );
	glRotatef(-angle_vert, 1.0, 0.0, 0.0 );
	drawBranchObject(len, thick);

	glRotatef(angle_vert, 1.0, 0.0, 0.0 );
		glRotatef(-b->angle_horiz - hoff, 0, 1, 0 );
	
	if (show_leaves && (b->twigs.size() == 0 || age_nowi(b->age) == 0)) {
		QMaths::srand(wind ? rndr(256) : b->random);

		draw_leaves(b);

		glRotatef(b->angle_horiz + hoff, 0, 1, 0 );
		glRotatef(-angle_vert, 1.0, 0.0, 0.0 );

	drawBranchObject(len*0.1, thick);

	glRotatef(angle_vert, 1.0, 0.0, 0.0 );
		glRotatef(-b->angle_horiz - hoff, 0, 1, 0 );
		
		draw_leaves(b);
	}

	for (list<Branch*>::iterator bi = b->twigs.begin(); bi != b->twigs.end(); bi++)
		draw_branch(*bi);

	glPopMatrix();
}

// yrs allows tree to be drawn as it was when it was younger
void Tree1::draw_tree(float yrs)
{	
	// global scaling and rotation already set up in calling routine

	if (yrs < 0.1 || yrs > age) yrs = age;
	/* When tree is created, age == 3 means 3 complete years of growth
	   i.e. 3 vertical segments of trunk and 2 layers of branches.
	   If draw_tree(3), we want to draw complete tree. If draw_tree(2.99),
	   this should look almost identical, so 3 means 100% of year 3 growth
	   and 2.99 means 99% of year 3 growth. */
	current_age = yrs;
	year_frac = current_age - int(current_age);
	
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	draw_branch(branch);
}
