/***************************************************************************
                          tree1.h 

This class draws a tree. The tree is represented by a recursive structure;
the Branch structure holds the size and position of one year's growth and
links to next year's offshoots. All random numbers are calculated when the
tree is constructed (except for the effects of wind), so that the tree can be
rotated efficiently.

                     -------------------
    begin                : Wed Oct 31 2001
    copyright            : (C) 2001 by tim
    email                : tim@simply.tim
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TREE1_H
#define TREE1_H

#include <list>

#include <qimage.h>
#include <qgl.h>

#include "tree.h"


struct Branch {
	float len;
	float thick;	// approx proportional to number of branches supported
	bool trunk;
	int depth;	// 0 = trunk, 1 = main branch, 2 = branch off branch etc.
	int age;
	int angle_vert;	// absolute - 90 is vertical, 0 is horizontal
	int angle_horiz;	// relative to previous branch; from above, 0 is east, 180 is west, 90 is south(?)
	int random;		// seed for random values used for leaves
	list <Branch*> twigs;
	
	~Branch();
	void add_twig(Branch* b) { if (b != NULL) twigs.push_back(b); }
};

/**represents a tree (i.e. branches and twigs)
  *@author tim
  */

class Tree1 : public Tree {
public: 
	Tree1(int age_, float len_);
	~Tree1();
	
	void draw_tree(float yrs=-1);		// if (yrs < age) draw tree as it was when it was younger

private:
	QImage tex1;	// texture for bark
	GLuint texnum;
	
	Branch* branch;	// vaguely fractal, i.e one branch + children = tree
	
	Branch* new_trunk(int age, float len);
	Branch* new_branch(int age, float len, int angle, int depth);
	void drawBranchObject(float len, float thick);
	void draw_branch(Branch* b);
	void draw_leaves(Branch* b);
};

#endif
